# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'main_ui.ui'
##
## Created by: Qt User Interface Compiler version 6.8.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QAbstractSpinBox, QApplication, QDoubleSpinBox, QFrame,
    QGroupBox, QLabel, QLayout, QLineEdit,
    QMainWindow, QPlainTextEdit, QPushButton, QScrollArea,
    QSizePolicy, QSpinBox, QToolBox, QWidget)

from pyqtgraph import PlotWidget

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(840, 400)
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.centralwidget.setMinimumSize(QSize(840, 400))
        self.settings = QScrollArea(self.centralwidget)
        self.settings.setObjectName(u"settings")
        self.settings.setGeometry(QRect(0, 20, 120, 380))
        self.settings.setMinimumSize(QSize(120, 380))
        self.settings.setFrameShape(QFrame.Shape.NoFrame)
        self.settings.setFrameShadow(QFrame.Shadow.Plain)
        self.settings.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
        self.settings.setWidgetResizable(True)
        self.scrollAreaWidgetContents = QWidget()
        self.scrollAreaWidgetContents.setObjectName(u"scrollAreaWidgetContents")
        self.scrollAreaWidgetContents.setGeometry(QRect(0, 0, 120, 380))
        self.toolBox = QToolBox(self.scrollAreaWidgetContents)
        self.toolBox.setObjectName(u"toolBox")
        self.toolBox.setEnabled(True)
        self.toolBox.setGeometry(QRect(1, 0, 118, 379))
        self.toolBox.setFrameShape(QFrame.Shape.NoFrame)
        self.sensorOptions = QWidget()
        self.sensorOptions.setObjectName(u"sensorOptions")
        self.sensorOptions.setEnabled(True)
        self.sensorOptions.setGeometry(QRect(0, 0, 118, 259))
        self.labPortName = QLabel(self.sensorOptions)
        self.labPortName.setObjectName(u"labPortName")
        self.labPortName.setGeometry(QRect(1, -1, 113, 20))
        self.setPortName = QLineEdit(self.sensorOptions)
        self.setPortName.setObjectName(u"setPortName")
        self.setPortName.setEnabled(True)
        self.setPortName.setGeometry(QRect(1, 19, 113, 20))
        self.labGaugeValue = QLabel(self.sensorOptions)
        self.labGaugeValue.setObjectName(u"labGaugeValue")
        self.labGaugeValue.setGeometry(QRect(1, 39, 113, 20))
        self.labNewtonPerCount = QLabel(self.sensorOptions)
        self.labNewtonPerCount.setObjectName(u"labNewtonPerCount")
        self.labNewtonPerCount.setGeometry(QRect(1, 79, 113, 20))
        self.butConnect = QPushButton(self.sensorOptions)
        self.butConnect.setObjectName(u"butConnect")
        self.butConnect.setGeometry(QRect(1, 119, 113, 22))
        self.butConnect.setCheckable(True)
        self.butConnect.setChecked(False)
        self.butConnect.setFlat(False)
        self.setNewtonPerCount = QDoubleSpinBox(self.sensorOptions)
        self.setNewtonPerCount.setObjectName(u"setNewtonPerCount")
        self.setNewtonPerCount.setEnabled(False)
        self.setNewtonPerCount.setGeometry(QRect(1, 99, 113, 20))
        self.setNewtonPerCount.setDecimals(12)
        self.setNewtonPerCount.setMinimum(-9999999999999999635896294965248.000000000000000)
        self.setNewtonPerCount.setMaximum(9999999999999999455752309870428160.000000000000000)
        self.setNewtonPerCount.setSingleStep(0.000001000000000)
        self.setNewtonPerCount.setStepType(QAbstractSpinBox.StepType.AdaptiveDecimalStepType)
        self.setNewtonPerCount.setValue(1.000000000000000)
        self.displayOptions = QGroupBox(self.sensorOptions)
        self.displayOptions.setObjectName(u"displayOptions")
        self.displayOptions.setGeometry(QRect(1, 139, 113, 119))
        font = QFont()
        font.setBold(True)
        self.displayOptions.setFont(font)
        self.setUnitDisplay = QLineEdit(self.displayOptions)
        self.setUnitDisplay.setObjectName(u"setUnitDisplay")
        self.setUnitDisplay.setEnabled(False)
        self.setUnitDisplay.setGeometry(QRect(1, 100, 109, 20))
        font1 = QFont()
        font1.setBold(False)
        self.setUnitDisplay.setFont(font1)
        self.setUnitDisplay.setMaxLength(8)
        self.setForceApplied = QDoubleSpinBox(self.displayOptions)
        self.setForceApplied.setObjectName(u"setForceApplied")
        self.setForceApplied.setEnabled(False)
        self.setForceApplied.setGeometry(QRect(1, 60, 109, 20))
        self.setForceApplied.setFont(font1)
        self.setForceApplied.setDecimals(8)
        self.setForceApplied.setMinimum(0.000000000000000)
        self.setForceApplied.setMaximum(999999.000000000000000)
        self.setForceApplied.setStepType(QAbstractSpinBox.StepType.AdaptiveDecimalStepType)
        self.setForceApplied.setValue(1.000000000000000)
        self.butTareDisplay = QPushButton(self.displayOptions)
        self.butTareDisplay.setObjectName(u"butTareDisplay")
        self.butTareDisplay.setEnabled(False)
        self.butTareDisplay.setGeometry(QRect(1, 19, 109, 22))
        self.butTareDisplay.setFont(font1)
        self.labForceApplied = QLabel(self.displayOptions)
        self.labForceApplied.setObjectName(u"labForceApplied")
        self.labForceApplied.setGeometry(QRect(1, 41, 109, 19))
        self.labForceApplied.setFont(font1)
        self.labUnitDisplay = QLabel(self.displayOptions)
        self.labUnitDisplay.setObjectName(u"labUnitDisplay")
        self.labUnitDisplay.setGeometry(QRect(1, 80, 109, 20))
        self.labUnitDisplay.setFont(font1)
        self.setGaugeValue = QSpinBox(self.sensorOptions)
        self.setGaugeValue.setObjectName(u"setGaugeValue")
        self.setGaugeValue.setEnabled(False)
        self.setGaugeValue.setGeometry(QRect(1, 59, 113, 20))
        self.setGaugeValue.setMinimum(-999999999)
        self.setGaugeValue.setMaximum(999999999)
        self.toolBox.addItem(self.sensorOptions, u"Sensor")
        self.moveOptions = QWidget()
        self.moveOptions.setObjectName(u"moveOptions")
        self.moveOptions.setGeometry(QRect(0, 0, 98, 28))
        self.labVelocity = QLabel(self.moveOptions)
        self.labVelocity.setObjectName(u"labVelocity")
        self.labVelocity.setGeometry(QRect(1, -1, 113, 20))
        self.setPosition = QSpinBox(self.moveOptions)
        self.setPosition.setObjectName(u"setPosition")
        self.setPosition.setGeometry(QRect(1, 81, 113, 20))
        self.setPosition.setMinimum(1)
        self.setPosition.setMaximum(46)
        self.butMove = QPushButton(self.moveOptions)
        self.butMove.setObjectName(u"butMove")
        self.butMove.setEnabled(False)
        self.butMove.setGeometry(QRect(1, 101, 113, 22))
        self.labPosition = QLabel(self.moveOptions)
        self.labPosition.setObjectName(u"labPosition")
        self.labPosition.setGeometry(QRect(1, 61, 113, 20))
        self.setVelocity = QSpinBox(self.moveOptions)
        self.setVelocity.setObjectName(u"setVelocity")
        self.setVelocity.setGeometry(QRect(1, 19, 113, 20))
        self.setVelocity.setMinimum(1)
        self.setVelocity.setMaximum(120)
        self.setVelocity.setValue(60)
        self.butUpdateVelocity = QPushButton(self.moveOptions)
        self.butUpdateVelocity.setObjectName(u"butUpdateVelocity")
        self.butUpdateVelocity.setEnabled(False)
        self.butUpdateVelocity.setGeometry(QRect(1, 39, 113, 22))
        self.GroupMovementPattern = QGroupBox(self.moveOptions)
        self.GroupMovementPattern.setObjectName(u"GroupMovementPattern")
        self.GroupMovementPattern.setGeometry(QRect(1, 123, 113, 120))
        self.GroupMovementPattern.setFont(font)
        self.GroupMovementPattern.setFlat(True)
        self.labStartPos = QLabel(self.GroupMovementPattern)
        self.labStartPos.setObjectName(u"labStartPos")
        self.labStartPos.setGeometry(QRect(1, 13, 111, 20))
        self.labStartPos.setFont(font1)
        self.setStartPos = QSpinBox(self.GroupMovementPattern)
        self.setStartPos.setObjectName(u"setStartPos")
        self.setStartPos.setGeometry(QRect(1, 33, 111, 20))
        self.setStartPos.setFont(font1)
        self.setStartPos.setMinimum(1)
        self.setStartPos.setMaximum(46)
        self.setStartPos.setValue(46)
        self.labEndPos = QLabel(self.GroupMovementPattern)
        self.labEndPos.setObjectName(u"labEndPos")
        self.labEndPos.setGeometry(QRect(1, 53, 111, 20))
        self.labEndPos.setFont(font1)
        self.setEndPos = QSpinBox(self.GroupMovementPattern)
        self.setEndPos.setObjectName(u"setEndPos")
        self.setEndPos.setGeometry(QRect(1, 73, 111, 20))
        self.setEndPos.setFont(font1)
        self.setEndPos.setMinimum(1)
        self.setEndPos.setMaximum(46)
        self.setEndPos.setValue(46)
        self.butSwapPositions = QPushButton(self.GroupMovementPattern)
        self.butSwapPositions.setObjectName(u"butSwapPositions")
        self.butSwapPositions.setGeometry(QRect(0, 94, 113, 22))
        self.butSwapPositions.setFont(font1)
        self.butSwapPositions.setLocale(QLocale(QLocale.English, QLocale.UnitedKingdom))
        self.toolBox.addItem(self.moveOptions, u"Movement")
        self.measurementOptions = QWidget()
        self.measurementOptions.setObjectName(u"measurementOptions")
        self.measurementOptions.setEnabled(True)
        self.measurementOptions.setGeometry(QRect(0, 0, 98, 28))
        self.fileLabel = QLabel(self.measurementOptions)
        self.fileLabel.setObjectName(u"fileLabel")
        self.fileLabel.setGeometry(QRect(1, -1, 113, 20))
        self.butFile = QPushButton(self.measurementOptions)
        self.butFile.setObjectName(u"butFile")
        self.butFile.setGeometry(QRect(1, 19, 113, 22))
        self.butFile.setCheckable(True)
        self.labLineSkips = QLabel(self.measurementOptions)
        self.labLineSkips.setObjectName(u"labLineSkips")
        self.labLineSkips.setGeometry(QRect(1, 41, 113, 20))
        self.labLineSkips.setFont(font1)
        self.setLineSkips = QSpinBox(self.measurementOptions)
        self.setLineSkips.setObjectName(u"setLineSkips")
        self.setLineSkips.setGeometry(QRect(1, 61, 113, 20))
        self.setLineSkips.setFont(font1)
        self.setLineSkips.setMinimum(0)
        self.setLineSkips.setMaximum(999999999)
        self.setLineSkips.setValue(0)
        self.setLineReads = QSpinBox(self.measurementOptions)
        self.setLineReads.setObjectName(u"setLineReads")
        self.setLineReads.setGeometry(QRect(1, 101, 113, 20))
        self.setLineReads.setFont(font1)
        self.setLineReads.setMinimum(1)
        self.setLineReads.setMaximum(999999999)
        self.setLineReads.setValue(1)
        self.labLineReads = QLabel(self.measurementOptions)
        self.labLineReads.setObjectName(u"labLineReads")
        self.labLineReads.setGeometry(QRect(1, 81, 113, 20))
        self.labLineReads.setFont(font1)
        self.timeLabel = QLabel(self.measurementOptions)
        self.timeLabel.setObjectName(u"timeLabel")
        self.timeLabel.setGeometry(QRect(1, 121, 113, 20))
        self.setTime = QDoubleSpinBox(self.measurementOptions)
        self.setTime.setObjectName(u"setTime")
        self.setTime.setGeometry(QRect(1, 141, 113, 20))
        self.setTime.setDecimals(6)
        self.setTime.setMinimum(0.000000000000000)
        self.setTime.setMaximum(9999999.000000000000000)
        self.setTime.setValue(0.000000000000000)
        self.toolBox.addItem(self.measurementOptions, u"Measurement")
        self.graphOptions = QWidget()
        self.graphOptions.setObjectName(u"graphOptions")
        self.graphOptions.setEnabled(True)
        self.graphOptions.setGeometry(QRect(0, 0, 98, 28))
        self.yLabel = QLineEdit(self.graphOptions)
        self.yLabel.setObjectName(u"yLabel")
        self.yLabel.setEnabled(True)
        self.yLabel.setGeometry(QRect(1, 59, 113, 20))
        self.yLabelLabel = QLabel(self.graphOptions)
        self.yLabelLabel.setObjectName(u"yLabelLabel")
        self.yLabelLabel.setGeometry(QRect(1, 39, 113, 20))
        self.xLimLabel = QLabel(self.graphOptions)
        self.xLimLabel.setObjectName(u"xLimLabel")
        self.xLimLabel.setGeometry(QRect(1, 79, 113, 20))
        self.title = QLineEdit(self.graphOptions)
        self.title.setObjectName(u"title")
        self.title.setGeometry(QRect(1, 19, 113, 20))
        self.titleLabel = QLabel(self.graphOptions)
        self.titleLabel.setObjectName(u"titleLabel")
        self.titleLabel.setGeometry(QRect(1, -1, 113, 20))
        self.plotTimerIntervalLabel = QLabel(self.graphOptions)
        self.plotTimerIntervalLabel.setObjectName(u"plotTimerIntervalLabel")
        self.plotTimerIntervalLabel.setGeometry(QRect(1, 219, 113, 20))
        self.setPlotTimerInterval = QSpinBox(self.graphOptions)
        self.setPlotTimerInterval.setObjectName(u"setPlotTimerInterval")
        self.setPlotTimerInterval.setGeometry(QRect(1, 239, 113, 20))
        self.setPlotTimerInterval.setMinimum(1)
        self.setPlotTimerInterval.setMaximum(100000)
        self.setPlotTimerInterval.setSingleStep(100)
        self.setPlotTimerInterval.setValue(200)
        self.xLimSet = QSpinBox(self.graphOptions)
        self.xLimSet.setObjectName(u"xLimSet")
        self.xLimSet.setGeometry(QRect(1, 99, 113, 20))
        self.xLimSet.setMinimum(-1)
        self.xLimSet.setMaximum(0)
        self.toolBox.addItem(self.graphOptions, u"Graph")
        self.settings.setWidget(self.scrollAreaWidgetContents)
        self.centerGraph = QWidget(self.centralwidget)
        self.centerGraph.setObjectName(u"centerGraph")
        self.centerGraph.setEnabled(True)
        self.centerGraph.setGeometry(QRect(120, 0, 600, 400))
        self.centerGraph.setMinimumSize(QSize(600, 400))
        self.graph1 = PlotWidget(self.centerGraph)
        self.graph1.setObjectName(u"graph1")
        self.graph1.setEnabled(True)
        self.graph1.setGeometry(QRect(0, 0, 600, 400))
        self.graph1.setFrameShape(QFrame.Shape.StyledPanel)
        self.graph1.setFrameShadow(QFrame.Shadow.Sunken)
        self.rightWidget = QWidget(self.centralwidget)
        self.rightWidget.setObjectName(u"rightWidget")
        self.rightWidget.setGeometry(QRect(720, 20, 120, 380))
        self.rightWidget.setMinimumSize(QSize(120, 380))
        self.butRecord = QPushButton(self.rightWidget)
        self.butRecord.setObjectName(u"butRecord")
        self.butRecord.setEnabled(False)
        self.butRecord.setGeometry(QRect(1, -1, 118, 24))
        self.butRecord.setCheckable(True)
        self.butSave = QPushButton(self.rightWidget)
        self.butSave.setObjectName(u"butSave")
        self.butSave.setEnabled(False)
        self.butSave.setGeometry(QRect(1, 47, 118, 24))
        self.butClear = QPushButton(self.rightWidget)
        self.butClear.setObjectName(u"butClear")
        self.butClear.setGeometry(QRect(1, 23, 118, 24))
        self.butSingleRead = QPushButton(self.rightWidget)
        self.butSingleRead.setObjectName(u"butSingleRead")
        self.butSingleRead.setEnabled(False)
        self.butSingleRead.setGeometry(QRect(1, 255, 118, 24))
        self.butSwitchManual = QPushButton(self.rightWidget)
        self.butSwitchManual.setObjectName(u"butSwitchManual")
        self.butSwitchManual.setEnabled(True)
        self.butSwitchManual.setGeometry(QRect(0, 339, 118, 40))
        self.butSwitchManual.setFont(font1)
        self.butForceStop = QPushButton(self.rightWidget)
        self.butForceStop.setObjectName(u"butForceStop")
        self.butForceStop.setEnabled(False)
        self.butForceStop.setGeometry(QRect(1, 119, 118, 24))
        self.butTare = QPushButton(self.rightWidget)
        self.butTare.setObjectName(u"butTare")
        self.butTare.setEnabled(False)
        self.butTare.setGeometry(QRect(1, 71, 118, 24))
        self.butTare.setCheckable(True)
        self.butHome = QPushButton(self.rightWidget)
        self.butHome.setObjectName(u"butHome")
        self.butHome.setEnabled(False)
        self.butHome.setGeometry(QRect(1, 95, 118, 24))
        self.SRCount = QLineEdit(self.rightWidget)
        self.SRCount.setObjectName(u"SRCount")
        self.SRCount.setGeometry(QRect(1, 279, 118, 20))
        self.SRCount.setReadOnly(True)
        self.settingsLabel = QLabel(self.centralwidget)
        self.settingsLabel.setObjectName(u"settingsLabel")
        self.settingsLabel.setGeometry(QRect(0, 0, 120, 20))
        self.settingsLabel.setMinimumSize(QSize(120, 20))
        self.buttonsLabel = QLabel(self.centralwidget)
        self.buttonsLabel.setObjectName(u"buttonsLabel")
        self.buttonsLabel.setGeometry(QRect(721, 0, 120, 20))
        self.buttonsLabel.setMinimumSize(QSize(120, 20))
        self.MDM = QWidget(self.centralwidget)
        self.MDM.setObjectName(u"MDM")
        self.MDM.setEnabled(True)
        self.MDM.setGeometry(QRect(121, 0, 598, 400))
        self.MDM.setMinimumSize(QSize(598, 400))
        self.MDM.setAutoFillBackground(True)
        self.labMDM = QLabel(self.MDM)
        self.labMDM.setObjectName(u"labMDM")
        self.labMDM.setGeometry(QRect(0, 0, 600, 20))
        self.labMDM.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.butReadForceMDM = QPushButton(self.MDM)
        self.butReadForceMDM.setObjectName(u"butReadForceMDM")
        self.butReadForceMDM.setEnabled(False)
        self.butReadForceMDM.setGeometry(QRect(120, 319, 238, 80))
        self.extraSettingsBoxMDM = QGroupBox(self.MDM)
        self.extraSettingsBoxMDM.setObjectName(u"extraSettingsBoxMDM")
        self.extraSettingsBoxMDM.setGeometry(QRect(1, 20, 118, 298))
        self.extraSettingsBoxMDM.setFont(font)
        self.labelFilenameMDM = QLabel(self.extraSettingsBoxMDM)
        self.labelFilenameMDM.setObjectName(u"labelFilenameMDM")
        self.labelFilenameMDM.setGeometry(QRect(1, 13, 113, 20))
        self.labelFilenameMDM.setFont(font1)
        self.labStepSizeMDM = QLabel(self.extraSettingsBoxMDM)
        self.labStepSizeMDM.setObjectName(u"labStepSizeMDM")
        self.labStepSizeMDM.setGeometry(QRect(1, 53, 113, 20))
        self.labStepSizeMDM.setFont(font1)
        self.butFileMDM = QPushButton(self.extraSettingsBoxMDM)
        self.butFileMDM.setObjectName(u"butFileMDM")
        self.butFileMDM.setGeometry(QRect(1, 33, 113, 22))
        self.butFileMDM.setFont(font1)
        self.butFileMDM.setCheckable(True)
        self.setStepSizeMDM = QDoubleSpinBox(self.extraSettingsBoxMDM)
        self.setStepSizeMDM.setObjectName(u"setStepSizeMDM")
        self.setStepSizeMDM.setGeometry(QRect(1, 73, 113, 20))
        self.setStepSizeMDM.setFont(font1)
        self.setStepSizeMDM.setDecimals(12)
        self.setStepSizeMDM.setMinimum(0.000000000000000)
        self.setStepSizeMDM.setMaximum(99.989999999999995)
        self.setStepSizeMDM.setSingleStep(0.000001000000000)
        self.setStepSizeMDM.setStepType(QAbstractSpinBox.StepType.AdaptiveDecimalStepType)
        self.setStepSizeMDM.setValue(0.050000000000000)
        self.yLabelLabel_2 = QLabel(self.extraSettingsBoxMDM)
        self.yLabelLabel_2.setObjectName(u"yLabelLabel_2")
        self.yLabelLabel_2.setGeometry(QRect(1, 213, 113, 20))
        self.yLabelLabel_2.setFont(font1)
        self.xLabel_2 = QLineEdit(self.extraSettingsBoxMDM)
        self.xLabel_2.setObjectName(u"xLabel_2")
        self.xLabel_2.setEnabled(True)
        self.xLabel_2.setGeometry(QRect(1, 273, 113, 20))
        self.xLabel_2.setFont(font1)
        self.yLabel_2 = QLineEdit(self.extraSettingsBoxMDM)
        self.yLabel_2.setObjectName(u"yLabel_2")
        self.yLabel_2.setEnabled(True)
        self.yLabel_2.setGeometry(QRect(1, 233, 113, 20))
        self.yLabel_2.setFont(font1)
        self.xLabelLabel_2 = QLabel(self.extraSettingsBoxMDM)
        self.xLabelLabel_2.setObjectName(u"xLabelLabel_2")
        self.xLabelLabel_2.setGeometry(QRect(1, 253, 113, 20))
        self.xLabelLabel_2.setFont(font1)
        self.title_2 = QLineEdit(self.extraSettingsBoxMDM)
        self.title_2.setObjectName(u"title_2")
        self.title_2.setEnabled(True)
        self.title_2.setGeometry(QRect(1, 193, 113, 20))
        self.title_2.setFont(font1)
        self.titleLabel_2 = QLabel(self.extraSettingsBoxMDM)
        self.titleLabel_2.setObjectName(u"titleLabel_2")
        self.titleLabel_2.setGeometry(QRect(1, 173, 113, 20))
        self.titleLabel_2.setFont(font1)
        self.graphMDM = PlotWidget(self.MDM)
        self.graphMDM.setObjectName(u"graphMDM")
        self.graphMDM.setEnabled(True)
        self.graphMDM.setGeometry(QRect(120, 20, 477, 298))
        self.graphMDM.setFrameShape(QFrame.Shape.StyledPanel)
        self.graphMDM.setFrameShadow(QFrame.Shadow.Plain)
        self.butSwitchDirectionMDM = QPushButton(self.MDM)
        self.butSwitchDirectionMDM.setObjectName(u"butSwitchDirectionMDM")
        self.butSwitchDirectionMDM.setEnabled(False)
        self.butSwitchDirectionMDM.setGeometry(QRect(1, 319, 118, 80))
        self.plainTextEdit = QPlainTextEdit(self.MDM)
        self.plainTextEdit.setObjectName(u"plainTextEdit")
        self.plainTextEdit.setGeometry(QRect(359, 319, 238, 56))
        self.plainTextEdit.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOn)
        self.plainTextEdit.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
        self.plainTextEdit.setReadOnly(True)
        self.butDeletePreviousMDM = QPushButton(self.MDM)
        self.butDeletePreviousMDM.setObjectName(u"butDeletePreviousMDM")
        self.butDeletePreviousMDM.setEnabled(False)
        self.butDeletePreviousMDM.setGeometry(QRect(359, 375, 238, 24))
        self.graphMDM.raise_()
        self.labMDM.raise_()
        self.butReadForceMDM.raise_()
        self.extraSettingsBoxMDM.raise_()
        self.butSwitchDirectionMDM.raise_()
        self.plainTextEdit.raise_()
        self.butDeletePreviousMDM.raise_()
        MainWindow.setCentralWidget(self.centralwidget)
        self.settings.raise_()
        self.rightWidget.raise_()
        self.settingsLabel.raise_()
        self.buttonsLabel.raise_()
        self.centerGraph.raise_()
        self.MDM.raise_()

        self.retranslateUi(MainWindow)

        self.toolBox.setCurrentIndex(0)
        self.butConnect.setDefault(False)


        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"Use the Force!", None))
        self.labPortName.setText(QCoreApplication.translate("MainWindow", u"Port Name", None))
        self.setPortName.setText(QCoreApplication.translate("MainWindow", u"No Port Found", None))
        self.labGaugeValue.setText(QCoreApplication.translate("MainWindow", u"Gauge Value", None))
        self.labNewtonPerCount.setText(QCoreApplication.translate("MainWindow", u"Load per Count", None))
        self.butConnect.setText(QCoreApplication.translate("MainWindow", u"Connect", None))
        self.displayOptions.setTitle(QCoreApplication.translate("MainWindow", u"Display Options", None))
        self.setUnitDisplay.setPlaceholderText(QCoreApplication.translate("MainWindow", u" mN", None))
        self.butTareDisplay.setText(QCoreApplication.translate("MainWindow", u"Tare Display Value", None))
        self.labForceApplied.setText(QCoreApplication.translate("MainWindow", u"True Load Value:", None))
        self.labUnitDisplay.setText(QCoreApplication.translate("MainWindow", u"Display Unit:", None))
        self.toolBox.setItemText(self.toolBox.indexOf(self.sensorOptions), QCoreApplication.translate("MainWindow", u"Sensor", None))
        self.labVelocity.setText(QCoreApplication.translate("MainWindow", u"Velocity", None))
        self.butMove.setText(QCoreApplication.translate("MainWindow", u"Move", None))
        self.labPosition.setText(QCoreApplication.translate("MainWindow", u"Position", None))
        self.butUpdateVelocity.setText(QCoreApplication.translate("MainWindow", u"Update Velocity", None))
        self.GroupMovementPattern.setTitle(QCoreApplication.translate("MainWindow", u"Movement Pattern", None))
        self.labStartPos.setText(QCoreApplication.translate("MainWindow", u"Start Position", None))
        self.labEndPos.setText(QCoreApplication.translate("MainWindow", u"End Position", None))
        self.butSwapPositions.setText(QCoreApplication.translate("MainWindow", u"Swap Positions", None))
        self.toolBox.setItemText(self.toolBox.indexOf(self.moveOptions), QCoreApplication.translate("MainWindow", u"Movement", None))
        self.fileLabel.setText(QCoreApplication.translate("MainWindow", u"File", None))
        self.butFile.setText(QCoreApplication.translate("MainWindow", u"-", None))
        self.labLineSkips.setText(QCoreApplication.translate("MainWindow", u"Line Skips", None))
        self.labLineReads.setText(QCoreApplication.translate("MainWindow", u"Line Reads (average)", None))
        self.timeLabel.setText(QCoreApplication.translate("MainWindow", u"Extra Duration (s)", None))
        self.toolBox.setItemText(self.toolBox.indexOf(self.measurementOptions), QCoreApplication.translate("MainWindow", u"Measurement", None))
        self.yLabel.setText(QCoreApplication.translate("MainWindow", u"Force [mN]", None))
        self.yLabel.setPlaceholderText(QCoreApplication.translate("MainWindow", u"Force [mN]", None))
        self.yLabelLabel.setText(QCoreApplication.translate("MainWindow", u"y-Label", None))
        self.xLimLabel.setText(QCoreApplication.translate("MainWindow", u"x-Scope", None))
        self.title.setPlaceholderText(QCoreApplication.translate("MainWindow", u"Force-Displacement Measurement", None))
        self.titleLabel.setText(QCoreApplication.translate("MainWindow", u"Title", None))
        self.plotTimerIntervalLabel.setText(QCoreApplication.translate("MainWindow", u"Refresh Time (ms)", None))
        self.toolBox.setItemText(self.toolBox.indexOf(self.graphOptions), QCoreApplication.translate("MainWindow", u"Graph", None))
        self.butRecord.setText(QCoreApplication.translate("MainWindow", u"Start", None))
        self.butSave.setText(QCoreApplication.translate("MainWindow", u"Save", None))
        self.butClear.setText(QCoreApplication.translate("MainWindow", u"Clear", None))
        self.butSingleRead.setText(QCoreApplication.translate("MainWindow", u"Single Read", None))
        self.butSwitchManual.setText(QCoreApplication.translate("MainWindow", u"Switch to Manual\n"
"Displacement Mode", None))
        self.butForceStop.setText(QCoreApplication.translate("MainWindow", u"Force Stop", None))
        self.butTare.setText(QCoreApplication.translate("MainWindow", u"Tare", None))
        self.butHome.setText(QCoreApplication.translate("MainWindow", u"Home", None))
        self.SRCount.setText(QCoreApplication.translate("MainWindow", u"0", None))
        self.settingsLabel.setText(QCoreApplication.translate("MainWindow", u"<html><head/><body><p align=\"center\"><span style=\" font-weight:700;\">Settings</span></p></body></html>", None))
        self.buttonsLabel.setText(QCoreApplication.translate("MainWindow", u"<html><head/><body><p align=\"center\"><span style=\" font-weight:700;\">Buttons</span></p></body></html>", None))
        self.labMDM.setText(QCoreApplication.translate("MainWindow", u"<html><head/><body><p align=\"center\"><span style=\" font-weight:700;\">Manual Displacement Mode</span></p></body></html>", None))
        self.butReadForceMDM.setText(QCoreApplication.translate("MainWindow", u"Read Force", None))
        self.extraSettingsBoxMDM.setTitle(QCoreApplication.translate("MainWindow", u"MDM Settings", None))
        self.labelFilenameMDM.setText(QCoreApplication.translate("MainWindow", u"Filename", None))
        self.labStepSizeMDM.setText(QCoreApplication.translate("MainWindow", u"Step Size (mm)", None))
        self.butFileMDM.setText(QCoreApplication.translate("MainWindow", u"-", None))
        self.yLabelLabel_2.setText(QCoreApplication.translate("MainWindow", u"y-Label", None))
        self.xLabel_2.setText(QCoreApplication.translate("MainWindow", u"Displacement [mm]", None))
        self.xLabel_2.setPlaceholderText(QCoreApplication.translate("MainWindow", u"Displacement [mm]", None))
        self.yLabel_2.setText(QCoreApplication.translate("MainWindow", u"Force [mN]", None))
        self.yLabel_2.setPlaceholderText(QCoreApplication.translate("MainWindow", u"Force [mN]", None))
        self.xLabelLabel_2.setText(QCoreApplication.translate("MainWindow", u"x-Label", None))
        self.title_2.setPlaceholderText(QCoreApplication.translate("MainWindow", u"Manual Force-Displacement Curve", None))
        self.titleLabel_2.setText(QCoreApplication.translate("MainWindow", u"Title", None))
        self.butSwitchDirectionMDM.setText(QCoreApplication.translate("MainWindow", u"Switch Direction", None))
        self.butDeletePreviousMDM.setText(QCoreApplication.translate("MainWindow", u"Delete Previous Reading", None))
    # retranslateUi

